﻿using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.BusinessLogicsContracts;
using IceCreamShopContracts.SearchModels;
using IceCreamShopContracts.StoragesContracts;
using IceCreamShopContracts.ViewModels;
using IceCreamShopDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IceCreamShopBusinessLogic.BusinessLogic
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;

        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }

        public List<ShopViewModel> ReadList(ShopSearchModel model)
        {
            _logger.LogInformation("ReadList. ShopName:{Name}. Id:{ Id}", model?.Name, model?.Id);
            var list = model == null ? _shopStorage.GetFullList() :
_shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;

        }

        public bool MakeSupply(ShopSearchModel model, IIceCreamModel iceCream, int count)
        {

            if (model == null)
                throw new ArgumentNullException(nameof(model));
            if (iceCream == null)
                throw new ArgumentNullException(nameof(iceCream));
            if (count <= 0)
                throw new ArgumentNullException("Количество должно быть положительным числом");

            ShopViewModel curModel = _shopStorage.GetElement(model);
            if (curModel == null)
                throw new ArgumentNullException(nameof(curModel));
            var countItems = curModel.ShopIceCreams.Select(x => x.Value.Item2).Sum();
            if (curModel.MaxCapacity - countItems >= count)
            {
                if (curModel.ShopIceCreams.TryGetValue(iceCream.Id, out var sameDocument))
                {
                    curModel.ShopIceCreams[iceCream.Id] = (iceCream, sameDocument.Item2 + count);
                    _logger.LogInformation("Same iceCream found by supply. Added {0} of {1} in {2} shop", count, iceCream.IceCreamName, curModel.ShopName);
                }
                else
                {
                    curModel.ShopIceCreams[iceCream.Id] = (iceCream, count);
                    _logger.LogInformation("New iceCream added by supply. Added {0} of {1} in {2} shop", count, iceCream.IceCreamName, curModel.ShopName);
                }
                _shopStorage.Update(new()
                {
                    Id = curModel.Id,
                    ShopName = curModel.ShopName,
                    Address = curModel.Address,
                    DateOpen = curModel.DateOpen,
                    ShopIceCreams = curModel.ShopIceCreams,
                    MaxCapacity = curModel.MaxCapacity
                });
            }
            else
            {
                _logger.LogWarning("Required shop is overflowed");
                return false;
            }
            return true;
        }

        public bool MakeSell(IIceCreamModel iceCream, int count)
        {
            return _shopStorage.SellIceCreams(iceCream, count);
        }

        public ShopViewModel ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. ShopName:{ShopName}.Id:{ Id}", model.Name, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(ShopBindingModel model, bool withParams =
true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.ShopName))
            {
                throw new ArgumentNullException("Нет названия магазина",
               nameof(model.ShopName));
            }
            if (string.IsNullOrEmpty(model.Address))
            {
                throw new ArgumentNullException("Нет адресса магазина",
               nameof(model.ShopName));
            }
            if(model.DateOpen == null)
            {
                throw new ArgumentNullException("Нет даты открытия магазина",
               nameof(model.ShopName));
            }
            _logger.LogInformation("Shop. ShopName:{ShopName}.Address:{Address}. DateOpen:{DateOpen}. Id: { Id}", model.ShopName, model.Address, model.DateOpen, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                Name = model.ShopName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }

    }
}
